# Feed Me!
#
# Copyrigt 2011 Mette & JB
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import random
import pygame, sys, os
from pygame.locals import *

# Game constants
GRIDSIZE = 32
WIDTH = 20
HEIGHT = 15
RIGHT = 0
UP = 1
LEFT = 2
DOWN = 3
NOTHING = 0
FLY = 1

# Initialize pygame variables
pygame.init()
window = pygame.display.set_mode((WIDTH*GRIDSIZE,HEIGHT*GRIDSIZE))
pygame.display.set_caption("Feed Me!")
screen = pygame.display.get_surface()

def load_image(name, colorkey=None):
    try:
        image = pygame.image.load(name)
    except pygame.error:
        print("Cannot load image:", name)
        raise SystemExit
    image = image.convert()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image

def load_sound(name):
    class NoneSound:
        def play(self): pass
    if not pygame.mixer:
        return NoneSound()
    try:
        sound = pygame.mixer.Sound(name)
    except pygame.error, message:
        print 'Cannot load sound:', wav
        raise SystemExit, message
    return sound

class TextManager:
    def __init__(self):
        self.font = pygame.font.Font("data/kongtext.ttf",16)
    def draw(self, surface, s, x, y):
        text = self.font.render(s,True,(0,0,0))
        text_rect = text.get_rect()
        text_rect.top= y * GRIDSIZE
        text_rect.left= x * GRIDSIZE
        surface.blit(text,text_rect) 

def read_hiscore():
    f = open("hiscore.txt")
    hiscore = int(f.readline())
    f.close()
    return hiscore

def write_hiscore(hiscore):
    f = open("hiscore.txt","w")
    f.write(str(hiscore))
    f.close
    
def draw_background(screen):
    screen.blit(img_background, img_background.get_rect())

class GameObject:
    def __init__(self, img, x=0, y=0):
        self.img = img
        self.x = x
        self.y = y
        self.frame = 0
        self.animation_speed = 30
        self.ctr_animation = 0
    def update(self):
        if self.img.__class__==list:
            self.ctr_animation += 1
            if self.ctr_animation == self.animation_speed:
                self.ctr_animation = 0
                self.frame += 1
                if self.frame == len(self.img):
                    self.frame = 0
    def draw(self, screen):
        if not self.img.__class__==list:
            img = self.img
        else:
            img = self.img[self.frame]
        rect = img.get_rect()
        rect.left = self.x * GRIDSIZE
        rect.top = self.y * GRIDSIZE
        screen.blit(img, rect)

class Player(GameObject):
    def __init__(self, img, x=0, y=0):
        self.is_jumping = False
        self.carries = NOTHING
        GameObject.__init__(self, img, x, y)

class Fly(GameObject):
    def __init__(self, img, x=0, y=0):
        self.direction = UP
        GameObject.__init__(self, img, x, y)

class Cat(GameObject):
    def __init__(self, img, x=0, y=0):
        self.direction = UP
        GameObject.__init__(self, img, x, y)

class Plant(GameObject):
    def __init__(self, img, x=0, y=0):
        self.is_jumping = False
        self.mode = 0
        GameObject.__init__(self, img, x, y)
    def draw(self, screen):
        if self.mode == 1:
            rect = img_feedme.get_rect()
            rect.left = self.x * GRIDSIZE
            rect.top = (self.y - 2) * GRIDSIZE
            screen.blit(img_feedme, rect)
        GameObject.draw(self, screen)
                      
class Game:
    def __init__(self):
        self.player = Player(img_player, 10, 11)
        self.fly = None
        self.ctr_jump_speed = 0
        self.jump_speed = 30
        
        self.ctr_fly = 0
        self.ctr_fly_speed = 0
        self.fly_speed = 20
        self.fly_destroy = 300
        self.fly_create = 120
        self.ctr_fly_create = 0
        
        self.plant = Plant([img_plant0, img_plant1], 18, 11)
        self.plant_hungry = 15
        self.ctr_plant_hungry = 0
        
        self.cat = Cat(img_cat, 13, 9)
        
        self.score = 0
        self.level = 1
        self.level_score = 0
        self.timer = 60
        self.ctr_timer = 0
        self.fps = 30
    def update(self):
        # Jump back down?
        self.ctr_jump_speed += 1
        if self.ctr_jump_speed == self.jump_speed:
            self.ctr_jump_speed = 0
            self.player.is_jumping = False
            self.player.y = 11
        # Create fly?
        self.ctr_fly += 1
        if self.ctr_fly == self.fly_create:
            # print "CREATE: " + str(self.ctr_fly)
            self.fly = Fly(img_flies, 4, 11)
        # Move fly
        self.ctr_fly_speed += 1
        if self.ctr_fly_speed > self.fly_speed and self.fly != None:
            self.ctr_fly_speed = 0
            # print "MOVE"
            if self.fly.direction == UP:
                self.fly.y -= 1
            if self.fly.direction == RIGHT and self.fly.x<8:
                self.fly.x += 1
            if self.fly.y == 8 and self.fly.direction == UP:
                self.fly.direction = RIGHT
        # Destroy fly?
        if self.ctr_fly == self.fly_destroy:
            # print "DESTROY: " + str(self.ctr_fly)
            self.ctr_fly = 0
            self.fly = None               
        # Update plant
        self.plant.update()
        # Eat?
        if self.player.x == 16 and not self.player.is_jumping:
            if self.player.carries == FLY:
                self.player.carries = NOTHING
                self.score += 5
                self.level_score += 1
                self.ctr_plant_hungry = 0
                self.plant.mode = 0
                snd_eat.play()
                if self.level_score == 3:
                    self.level_score = 0
                    self.level += 1
                    self.score += self.timer
                    self.timer = 60 - (self.level-1) * 5
                    return 1
        return 2
        
    def draw(self, screen):
        draw_background(screen)
        s = "SCORE: "+str(self.score) + " TIME: "+str(self.timer)
        text_manager.draw(screen, s, 0, 0)
        self.player.draw(screen)
        if self.fly != None:
            self.fly.draw(screen)
        self.plant.draw(screen)
        self.cat.draw(screen)
        
    def move_player(self, direction):
        if not self.player.is_jumping:
            if direction == RIGHT and self.player.x < 16:
                self.player.x += 2
                snd_move.play()
            elif direction == LEFT and self.player.x > 2:
                self.player.x -= 2
                snd_move.play()
    def jump(self):
        if not self.player.is_jumping: # Jump
            self.player.is_jumping = True
            self.ctr_jump_speed = 0
            self.player.y = 9
            snd_jump.play()
            if self.player.carries == NOTHING:
                if self.fly != None:
                    if self.fly.x == self.player.x or self.fly.x == (self.player.x + 1):
                        self.player.carries = FLY
                        self.fly = None
                        snd_pickup.play()
                  
# Initialize variables
img_background = load_image("data/background.png")
img_player = load_image("data/player.png", -1)
img_flies = load_image("data/flies.png", -1)
img_plant0 = load_image("data/plant0.png", -1)
img_plant1 = load_image("data/plant1.png", -1)
img_feedme = load_image("data/feedme.png", -1)
img_cat = load_image("data/cat.png", -1)

snd_move = load_sound("data/move.wav")
snd_jump = load_sound("data/jump.wav")
snd_pickup = load_sound("data/pickup.wav")
snd_eat = load_sound("data/eat.wav")
snd_hungry = load_sound("data/hungry.wav")

game_mode = 0
hiscore = read_hiscore()
game = None
text_manager = TextManager()
clock = pygame.time.Clock()
         
# Main game loop
while True:
    if game_mode == 0: # Title screen
        # Get Input
        for event in pygame.event.get():
            if event.type == QUIT:
                sys.exit(0)
            elif event.type == KEYDOWN:
                if event.key == K_ESCAPE:
                    sys.exit(0)
                elif event.key == K_SPACE:
                    game = Game()
                    game_mode = 1
        
        draw_background(screen)           
        text_manager.draw(screen, "FEED ME!", 8, 6)
        text_manager.draw(screen, "HISCORE: "+str(hiscore), 7, 8)
        text_manager.draw(screen, "PRESS <SPACE> TO START", 4, 10)
        pygame.display.flip()
        clock.tick(30)
  
    if game_mode == 1: # Ready?
        # Get Input
        for event in pygame.event.get():
            if event.type == QUIT:
                sys.exit(0)
            elif event.type == KEYDOWN:
                if event.key == K_ESCAPE:
                    sys.exit(0)
        
        draw_background(screen)
        text_manager.draw(screen, "LEVEL "+str(game.level), 8 , 5)         
        text_manager.draw(screen, "READY?", 8, 6)
        pygame.display.flip()
        pygame.time.delay(2000)
        game_mode = 2
    
    if game_mode == 2: # In-Game
        # Get Input
        for event in pygame.event.get():
            if event.type == QUIT:
                sys.exit(0)
            elif event.type == KEYDOWN:
                if event.key == K_ESCAPE:
                    sys.exit(0)
                elif event.key == K_LEFT:
                    game.move_player(LEFT)
                elif event.key == K_RIGHT:
                    game.move_player(RIGHT)
                elif event.key == K_SPACE:
                    game.jump()                    
                           
        game_mode = game.update()
            
        # Manage timer
        game.ctr_timer += 1
        if game.ctr_timer >= game.fps:
            game.ctr_timer = 0
            game.timer -= 1
            game.ctr_plant_hungry += 1
            if game.ctr_plant_hungry == game.plant_hungry:
                snd_hungry.play()
                game.ctr_plant_hungry = 0
                if game.plant.mode == 0: # GET HUNGRY!
                    game.plant.mode = 1
                    # print "HUNGRY!"
                elif game.plant.mode == 1: # EATS PLAYER!
                    game_mode = 3
                    # print "EATS PLAYER!"
                    
            if game.timer == 0: # OUT OF TIME
                game_mode = 3
            
        game.draw(screen)
        pygame.display.flip()        
        clock.tick(game.fps)
         
    if game_mode == 3: # Game over
        # Get Input
        for event in pygame.event.get():
            if event.type == QUIT:
                sys.exit(0)
            elif event.type == KEYDOWN:
                if event.key == K_ESCAPE:
                    sys.exit(0)
        
        draw_background(screen)           
        text_manager.draw(screen, "GAME OVER!", 8, 6)
        text_manager.draw(screen, "SCORE: "+str(game.score), 8, 7)
        if game.score > hiscore:
            hiscore = game.score
            write_hiscore(hiscore)
            text_manager.draw(screen, "NEW HISCORE!", 8, 8)
        pygame.display.flip()
        pygame.time.delay(2000)
        game_mode = 0
        game = None
